#!/usr/bin/env python3

# Support for the Digilent Cmod A7 Board
from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from gateware import info
from gateware import led
from gateware import spi_flash

from targets.utils import csr_map_update, period_ns


class _CRG(Module):
    def __init__(self, platform):
        self.clock_domains.cd_sys = ClockDomain()
        self.clock_domains.cd_clk200 = ClockDomain()
        clk12 = platform.request("clk12")
        rst = platform.request("user_btn", 0)
        self.specials += [
            Instance("BUFG", i_I=clk12, o_O=self.cd_sys.clk),
            AsyncResetSynchronizer(self.cd_sys, rst),
        ]


class BaseSoC(SoCCore):
    csr_peripherals = (
        "spiflash",
        "info",
    )
    csr_map_update(SoCCore.csr_map, csr_peripherals)

    mem_map = {
        "spiflash": 0x20000000,  # (default shadow @0xa0000000)
    }
    mem_map.update(SoCCore.mem_map)

    def __init__(self, platform, spiflash="spiflash_1x", **kwargs):
        if 'integrated_rom_size' not in kwargs:
            kwargs['integrated_rom_size']=0x8000
        if 'integrated_sram_size' not in kwargs:
            kwargs['integrated_sram_size']=0x8000

        clk_freq = int(100e6)
        SoCCore.__init__(self, platform, clk_freq, **kwargs)

        self.submodules.crg = _CRG(platform)
        self.crg.cd_sys.clk.attr.add("keep")
        self.platform.add_period_constraint(self.crg.cd_sys.clk, period_ns(clk_freq))

        # Basic peripherals
        self.submodules.info = info.Info(platform, self.__class__.__name__)

        # spi flash
        spiflash_pads = platform.request(spiflash)
        spiflash_pads.clk = Signal()
        self.specials += Instance("STARTUPE2",
                                  i_CLK=0, i_GSR=0, i_GTS=0, i_KEYCLEARB=0, i_PACK=0,
                                  i_USRCCLKO=spiflash_pads.clk, i_USRCCLKTS=0, i_USRDONEO=1, i_USRDONETS=1)
        spiflash_dummy = {
            "spiflash_1x": 9,
            "spiflash_4x": 11,
        }
        self.submodules.spiflash = spi_flash.SpiFlash(
                spiflash_pads,
                dummy=spiflash_dummy[spiflash],
                div=2)
        self.add_constant("SPIFLASH_PAGE_SIZE", 256)
        self.add_constant("SPIFLASH_SECTOR_SIZE", 0x10000)
        self.add_wb_slave(mem_decoder(self.mem_map["spiflash"]), self.spiflash.bus)
        self.add_memory_region(
            "spiflash", self.mem_map["spiflash"] | self.shadow_base, 16*1024*1024)


SoC = BaseSoC

# Build --------------------------------------------------------------------------------------------

def main():
    parser = argparse.ArgumentParser(description="LiteX SoC on cmoda7")
    builder_args(parser)
    args = parser.parse_args()

    cls = BaseSoC
    soc = cls(args)
    builder = Builder(soc, **builder_argdict(args))
    builder.build()


if __name__ == "__main__":
    main()



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
from enum import Enum
import obt.path 
import obt.command 

###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################
class Type(Enum):
  SINGLE = 1 # loose docker container
  COMPOSITE = 2  # compose based container set

###############################################################################

def module_class(module_path,dokname):
  import obt.module
  the_module = obt.module.instance("dok_"+dokname,module_path)
  if the_module != None:
    return the_module.dockerinfo
  return None 

###############################################################################

def docker_dirs():
  docker_dirs_list = list()
  module_dirs_list = os.environ["OBT_MODULES_PATH"].split(":")
  for module_dir in module_dirs_list:
    docker_path = obt.path.Path(module_dir)/"docker"
    if docker_path.exists():
      docker_dirs_list += [docker_path]
  return docker_dirs_list

###############################################################################

def descriptor(dokname):
  docker_dirs_list = docker_dirs()
  for docker_dir in docker_dirs_list:
    try_path = docker_dir/dokname
    if try_path.exists():
      try_path2 = try_path/("%s.py"%dokname)
      if try_path2.exists():
        return module_class(try_path2,dokname)()

###############################################################################

def enumerate():
  #######################################
  class EnumItem:
    def __init__(self,name,fullpath,module):
      self._name = name 
      self._fullpath = fullpath
      self._module = module
  #######################################
  module_dict = dict()
  module_dirs_list = os.environ["OBT_MODULES_PATH"].split(":")
  for module_dir in module_dirs_list:
    #print(dep_repo)
    docker_path = obt.path.Path(module_dir)/"docker"
    if docker_path.exists():
      path_list = os.listdir(docker_path)
      for item in path_list:
        module_test_path = docker_path/item/("%s.py"%item)
        if module_test_path.exists():
          mclass = module_class(module_test_path,item)
          module = mclass()
          if module!=None:
            e = EnumItem(item,module_test_path,module)
            module_dict[item] = e
  return module_dict

###############################################################################

def enumerate_all_images():
  cmdlist = ["docker","images","-aq"]
  cmd = obt.command.Command(cmdlist)
  as_str = cmd.capture()
  as_list = as_str.split("\n")
  # remove empty items
  as_list = [ x for x in as_list if x ]
  return as_list

def enumerate_all_continaers():
  cmdlist = ["docker","ps","-aq"]
  cmd = obt.command.Command(cmdlist)
  as_str = cmd.capture()
  as_list = as_str.split("\n")
  as_list = [ x for x in as_list if x ]
  return as_list



from obt import pathtools

def try_update(content_path=None,manifest_path=None,update_method=None,force=False):
  md5_of_data = pathtools.md5_of_file(content_path)
  touch_manifest = False
  ###########################
  if manifest_path.exists():
    fil = open(str(manifest_path),"rb")
    manifest_md5 = fil.readline().decode("utf-8")
    fil.close()
    if md5_of_data != manifest_md5:
      touch_manifest = True
  else:
    touch_manifest = True
  ###########################
  if touch_manifest or force:
    update_method()
    # write md5 to manifest
    fil = open(str(manifest_path),"w")
    fil.write(md5_of_data)
    fil.close()
  ###########################


def try_update_on_content_list(content_paths=None,manifest_path=None,update_method=None,force=False):
  md5_of_data = pathtools.md5_of_files(content_paths)
  touch_manifest = False
  ###########################
  if manifest_path.exists():
    fil = open(str(manifest_path),"rb")
    manifest_md5 = fil.readline().decode("utf-8")
    fil.close()
    if md5_of_data != manifest_md5:
      touch_manifest = True
  else:
    touch_manifest = True
  ###########################
  if touch_manifest or force:
    update_method()
    # write md5 to manifest
    fil = open(str(manifest_path),"w")
    fil.write(md5_of_data)
    fil.close()
  ###########################




#!/usr/bin/env python3

import os, sys, subprocess, pathlib
import ci_common

########################################################

this_dir = os.path.dirname(os.path.realpath(__file__))
this_dir = pathlib.Path(os.path.abspath(this_dir))

orkid_dir = None
if "ORKID_WORKSPACE_DIR" in os.environ:
  orkid_dir = pathlib.Path(os.environ["ORKID_WORKSPACE_DIR"])

########################################################
# we need an ssh key for the worker to git clone with
#  use a CICD key, not your personal key, as it could
#  be exposed by the build of products
########################################################

secrets_dir = os.environ["SECRETSDIR"]
private_key = secrets_dir/"buildkey.rsa"
public_key = secrets_dir/"buildkey.rsa.pub"

########################################################

cmdlist = list()

cmdlist += ["docker","run"]

cmdlist += ['-v',"%s:/home/workerub20/.ssh/id_rsa:ro"%str(private_key)]
cmdlist += ['-v',"%s:/home/workerub20/.ssh/id_rsa.pub:ro"%str(public_key)]

if orkid_dir!=None:
  cmdlist += ['-v',"%s:/home/workerub20/orkid"%str(orkid_dir)]

cmdlist += ['-it','obtcicd/worker_focal','/bin/bash']

########################################################

ci_common.sync_subprocess(cmdlist)



#!/usr/bin/env python3
import jinja2, git, datetime, os, subprocess, argparse, shlex, logging, sys, threading, time
from http import server
from yarl import URL
from pathlib import Path
import badge, _masterimpl, uuid, urllib
from enum import Enum
from orkbb import Branch, BranchStatus
from copy import deepcopy
################################################################################

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

class RepoWatcher:
  def __init__(self,config,repo_obj):
    self.config = config
    self.repo_obj = repo_obj
    self.commit_list = list()
    self.branch_dict = dict()
    self.GITDIR = config.repobasedir/self.repo_obj.subdir
    self.giturl = URL(repo_obj.giturl)
    self.update_count = 0
    self.watcherlock = threading.Lock()
    print("OUTPUTDIR<%s>"%self.config.outputdir)
    print("self.GITDIR<%s>"%self.GITDIR)
    ################################################################################
    # watcher repo setup
    ################################################################################
    self.watcherlock.acquire(blocking=True)

    print("begin git clone...")
    if self.repo_obj.skipLFS:
      os.environ["GIT_LFS_SKIP_SMUDGE"]="1"
    os.system("git clone --mirror %s %s"%(self.giturl,self.GITDIR))
    print("end git clone...")

    self.gitrepo = git.Repo(self.GITDIR)
    self.watcherlock.release()

  ################################################################################
  #
  ################################################################################

  def get_branch(self,named):
    item = None
    self.watcherlock.acquire(blocking=True)
    #print("repo<%s> brdict<%s>"%(self.repo_obj.name,self.branch_dict))
    if named in self.branch_dict:
      item = self.branch_dict[named]
    self.watcherlock.release()
    return item

  ################################################################################
  #
  ################################################################################

  def _extract_commit(self,commit):
    sha = commit.hexsha
    email = commit.committer.email
    date = datetime.datetime.fromtimestamp(commit.committed_date)
    return {
      "sha": sha,
      "author": email,
      "date": date,
      "commitpage": self.giturl/"commit"/sha }

  ################################################################################
  def getBranch(self,named):
    branch = None
    self.watcherlock.acquire(blocking=True)
    if named in self.branch_dict:
      branch = self.branch_dict[named]
    self.watcherlock.release()
    return branch
  ################################################################################
  # enumerate commits
  # enumerate branches
  ################################################################################
  def update_branch_info(self):
    self.watcherlock.acquire(blocking=True)
    first_update = (self.update_count==0)
    ##########################################
    #print("begin refetch (self.update_count<%d>)...."%self.update_count)
    for remote in self.gitrepo.remotes:
      remote.fetch()
    #print("end refetch....")
    ##########################################
    self.commit_list.clear()
    ##########################################
    #print("begin parse commits....")
    raw_commit_list = self.gitrepo.iter_commits('--all', max_count=10)
    for commit in raw_commit_list:
      item = self._extract_commit(commit)
      self.commit_list += [ item ]
    #print(self.commit_list)
    #print("end parse commits....")
    ##########################################
    #print("begin branches....")
    for branch_id in self.gitrepo.branches:
      branch_id = branch_id.name
      #print(branch_id)
      ###################################
      # fetch previous, or create new branch_item
      ###################################
      if branch_id in self.branch_dict.keys():
        branch_item = self.branch_dict[branch_id]
        #branch_item["prev_status"] = branch_item["status"]
      else:
        branch_item = Branch()
        branch_item.name = branch_id
        branch_item.repo_obj = self.repo_obj
        branch_item.remote_giturl = self.giturl/"tree"/branch_id
        self.branch_dict[branch_id] = branch_item
        #print("add branch id<%s>"%branch_id)
      ###################################
      # update branch_item
      ###################################
      branch_commit_list = list()
      branch_commits = self.gitrepo.iter_commits(max_count=2, rev=branch_id)
      for commit in branch_commits:
        branch_commit_list += [ self._extract_commit(commit) ]
      if len(branch_commit_list)>0:
        sha = (branch_commit_list[0])["sha"]
        branch_item.last_sha = sha
        branch_item.last_author = (branch_commit_list[0])["author"]
        branch_item.last_date = (branch_commit_list[0])["date"]
        branch_item.last_commit_url = self.giturl/"commit"/sha
    #print(self.branch_dict)
    #print("end parse branches....")
    self.update_count += 1
    self.watcherlock.release()

"""
   def __watch_loop(self):
  while True:
   update_branch_info()
   #update_site()
   ######################################
   # refresh branch symlinks
   ######################################
   for key in self.branch_dict.keys():
     branch_item = self.branch_dict[key]
     SHA = branch_item["lastsha"]
     name = branch_item["name"]
     linksrcpath = self.BUILDSBASEDIR/name
     linkdstpath = self.BUILDSBASEDIR/SHA
     os.system("rm -f %s"%linksrcpath)
     os.chdir(self.BUILDSBASEDIR)
     os.system("ln -s %s %s"%(SHA,name.replace("/","_")))
     STDOUTHTML = self.BUILDSBASEDIR/SHA/"stdout.html"
     if False==os.path.exists(STDOUTHTML):
       os.system("touch %s"%STDOUTHTML)
   ######################################
   # refresh builds
   ######################################
   for key in self.branch_dict.keys():
     branch_item = self.branch_dict[key]
     SHA = branch_item["lastsha"]
     ###########################################
     branch_item["status"] = orkbb.BranchStatus.BUILDING
     update_site()
     ###########################################
     print("building branch<%s> @ sha<%s>"%(branch_item["fixed_branch_name"],branch_item["lastsha"]))
     ###################################################
     name = branch_item["name"]
     os.chdir(SCRIPTDIR)
     ###################################################
     update_site()
   ######################################
   # rerun templatng with build status
   ######################################
   #update_site()
   ######################################
   # sleep
   ######################################
   #print("Sleeping for <%d> seconds" % config.sleeptime)
"""
